@@ -39,6 +39,8 @@
 #define ALLOW_NOESCAPE PACKAGE_DATA_DIR \
             G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "allow_noescape"
 
+#define MAX_WINDOW_SIZE (G_MAXUINT16 / 2)
+
 #ifdef ENABLE_NLS
 #  include <libintl.h>
 #  define _(String) gettext(String)
@@ -401,6 +403,28 @@ cb_timeout (gint *timeout)
 }
 
 
+gboolean
+label_width_okay (const gchar *str)
+{
+	GtkWidget *dummy;
+	PangoLayout *layout;
+	int width;
+
+	dummy = gtk_label_new (NULL);
+	layout = gtk_label_get_layout (GTK_LABEL(dummy));
+	pango_layout_set_text (layout, str, -1);
+	pango_layout_get_pixel_size (layout, &width, NULL);
+	gtk_widget_destroy (dummy);
+
+	if (width > MAX_WINDOW_SIZE) {
+		g_warning ("%s: button too wide\n", PACKAGE);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
 void
 window_create (void)
 {
@@ -547,12 +571,21 @@ window_create (void)
 					s++;
 				}
 				s = g_strconcat ("gtk-", p, NULL);
+				if (!label_width_okay (s)) {
+					/* XXX: Truncate and carry on, or quit? */
+					prog_cleanup ();
+				}
 				/* XXX: gtk_button_new_from_stock deprecated since GTK 3.10 */
 				btn = gtk_button_new_from_stock (s);
 				g_free (s);
 				g_free (p);
 			}
 			else {
+
+				if (!label_width_okay (button->label)) {
+					/* XXX: Truncate and carry on, or quit? */
+					prog_cleanup ();
+				}
 				btn = gtk_button_new_with_mnemonic (button->label);
 			}
 
@@ -616,14 +649,12 @@ window_create (void)
 	}
 
 	gtk_window_set_position (GTK_WINDOW(window), gx.window_position);
+	gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
 
 	if (gx.geom_str != NULL) {
 		gtk_widget_show_all (vbox); /* must precede parse_geometry */
 		gtk_window_parse_geometry (GTK_WINDOW(window), gx.geom_str);
 	}
-	else {
-		gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
-	}
 
 	/* open the window */
 	gtk_widget_show_all (window);
